/* GCC */
#include <fstream>
#include <vector>
#include <string>
#include <algorithm>
#include <map>

using namespace std;
typedef unsigned long long uint64;
struct Word
{
	Word(uint64 c, int w)
		: code(c), weight(w)
	{}
	uint64 code;
	int weight;
};

int n, m, prefix_digits, suffix_digits, prefix_count, suffix_count,
	p[10][16], last_idx, su_weight;
uint64 k;
vector<Word> prefix, suffix, *gen_out;
vector<int> prefix_weight;

inline bool operator<(const Word& w1, const Word& w2)
{
	if(w1.weight < w2.weight) return true;
	if(w1.weight > w2.weight) return false;
	return w1.code < w2.code;
}

inline int power(int base, int exp)
{
	int res(1);
    while(exp--)
		res *= base;
	return res;
}

inline int count_less(const vector<Word>& words, int weight)
{
    return lower_bound(words.begin(), words.end(), Word(0, weight)) - words.begin();
}

inline int count_equal(const vector<Word>& words, int weight)
{
	return count_less(words, weight + 1) - count_less(words, weight);
}

inline int count_greater(const vector<Word>& words, int weight)
{
	return words.size() - count_less(words, weight + 1);
}

void gen(int idx, uint64 cur = 0, int weight = 0) // [idx, last_idx)
{
    if(idx == last_idx)
	{
		gen_out->push_back(Word(cur, weight));
		return;
	}
	cur *= n;
	for(int i = 0; i < n; ++i)
		gen(idx + 1, cur + i, weight + p[idx][i]);
}

// Count how many words have weight strictly greater than 'weight'
uint64 f(int weight)
{
	static map<int, uint64> fs;
	static map<int, uint64>::iterator fs_it;
	// Check whether we computed this query before
	if((fs_it = fs.find(weight)) != fs.end())
		return fs_it->second;
	// If not, compute it and save result to fs
    uint64 res(0);
    for(int i = 0; i < prefix_count; ++i)
	{
		if(prefix[i].weight > weight)
			break;
		res += count_greater(suffix, weight - prefix[i].weight);
	}
	fs[weight] = res;
    return res;
}

// Decode word from code to string
string decode(uint64 code)
{
    string ans;
    for(int i = 0; i < m; ++i, code /= n)
    	ans.push_back('a' + (char)(code % n));
    reverse(ans.begin(), ans.end());
    return ans;
}

int main(void)
{
	// Input
    ifstream in("aliens.in");
    in >> n >> m >> k;
    for(int i = 0; i < m; ++i)
		for(int j = 0; j < n; ++j)
			in >> p[i][j];
    in.close();
    prefix_digits = m / 2, suffix_digits = m - prefix_digits;
    prefix.reserve(power(n, prefix_digits));
    suffix.reserve(power(n, suffix_digits));
    // Generate prefixes
    gen_out = &prefix, last_idx = prefix_digits;
    gen(0);
    prefix_count = prefix.size();
    prefix_weight.resize(prefix_count);
    // Generate suffixes
    gen_out = &suffix, last_idx = m;
    gen(prefix_digits);
    suffix_count = suffix.size();
    // Sort them according to weight and lexicographically if weights are the same
    sort(prefix.begin(), prefix.end());
    sort(suffix.begin(), suffix.end());
    // Weights of prefixes
    for(int i = 0; i < prefix_count; ++i)
		prefix_weight[prefix[i].code] = prefix[i].weight;
    // Binary search to find the weight of k'th word(r)
    int // (l; r]
		l = prefix.front().weight + suffix.front().weight - 1,
		r = prefix.back().weight + suffix.back().weight, c;
    while(r - l > 1)
	{
		c = (l + r) >> 1;
		if(f(c) < k)
			r = c;
		else
			l = c;
	}
	// Recover k'th word, which, lexicographically
	// sorted, has index idx(starting from 1)
	uint64 idx = k - f(r), cur_idx(0), cur_eq, word;
	for(int pr = 0; pr < prefix_count; ++pr)
	{
		su_weight = r - prefix_weight[pr];
        cur_eq = count_equal(suffix, su_weight);
        if(cur_idx + cur_eq < idx)
		{
			cur_idx += cur_eq;
			continue;
		}
        word = (uint64)(pr) * (uint64)(suffix_count) +
			suffix[count_less(suffix, su_weight) + idx - cur_idx - 1].code;
		break;
	}
	// Output
    ofstream out("aliens.out");
    out << decode(word) << '\n';
    out.close();
    return 0;
}
